/**
 * 
 */
package net.javamonkey.syringe.models;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Represents the full dependency configuration.
 * 
 * @author Matthew Cory
 * 
 */
public final class Context {

	public Collection<Dependency> getDependencies() {
		if (dependencies == null) {
			return null;
		}
		
		return dependencies.values();
	}

	public void setDependencies(Collection<Dependency> dependencies) {
		this.dependencies = new HashMap<String, Dependency>(dependencies.size());
		for (final Dependency dep : dependencies) {
			addDependency(dep);
		}
	}
	
	public void addDependency(final Dependency dependency) {
		if (dependencies == null) {
			dependencies = new HashMap<String, Dependency>();
		}
		
		dependencies.put(dependency.getId(), dependency);
	}
	
	public Dependency getDependencyById(final String id) {
		if (dependencies == null) {
			log.warn("Context -> Dependencies were null.");
			return null;
		}
		
		if (!(dependencies.containsKey(id))) {
			log.warn("Context -> Dependencies did not contain id " + id);
			return null;
		}
		
		return dependencies.get(id);
	}

	private Map<String, Dependency> dependencies;
	private static final Logger log = LoggerFactory.getLogger(Context.class);
}
