/**
 * 
 */
package net.javamonkey.syringe.utils.type;

import java.lang.reflect.Method;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import net.javamonkey.syringe.models.Argument;

/**
 * @author Matthew
 *
 */
public final class MethodFinder {
	private MethodFinder() { }
	
	public static Method findMethod(final Class<?> klass, final String name, final Collection<Argument> arguments) {
		Method res = null;
		
		final Set<Method> nameAndCountPossibles = new HashSet<Method>();
		
		final Method[] methods = klass.getMethods();
		for (final Method method : methods) {
			final String methodName = method.getName();
			if (!(methodName.equals(name))) {
				continue;
			}
			
			final int numParams = method.getParameterTypes().length;
			final int numArgs = arguments.size();
			if (numParams != numArgs) {
				continue;
			}
			
			nameAndCountPossibles.add(method);
		}
		
		// All of the name-and-count possible methods now have the same name and the 
		// same number of arguments.  Now we need to check the actual parameter types 
		// and see if the arguments match.
		final Set<Method> typePossibles = new HashSet<Method>();
		for (final Method ncPoss : nameAndCountPossibles) {
			final Class<?>[] params = ncPoss.getParameterTypes();
			final int numParams = params.length;
			boolean isMatch = true;
			for (int i = 0; i < numParams; ++i) {
				final Argument arg = findArgumentByIndex(arguments, i);
				if (arg == null) {
					isMatch = false;
					break;
				}
				
				final Class<?> paramType = params[i];
				if (!TypeConverter.canConvert(arg.getStrValue(), paramType)) {
					isMatch = false;
					break;
				}
			}
			
			if (!isMatch) {
				continue;
			}
			
			typePossibles.add(ncPoss);
		}
		
		// typePossibles should only contain a single
		// method; if not, though, let's just get the
		// first one.
		for (final Method m : typePossibles) {
			res = m;
			break;
		}
		
		return res;
	}
	
	private static Argument findArgumentByIndex(final Collection<Argument> arguments, final int index) {
		for (final Argument arg : arguments) {
			if (arg.getIndex() == index) {
				return arg;
			}
		}
		
		return null;
	}
}
