/**
 * 
 */
package net.javamonkey.syringe.utils.type;

import java.util.Collection;
import java.util.HashSet;

/**
 * Utility class to convert string values to various
 * primitive types.  This class knowingly breaks some
 * design principles (i.e. Open/Closed) due to the
 * stability of Java primitives and to just simply
 * make things easier.
 * 
 * @author Matthew Cory
 *
 */
public final class PrimitiveConverter {
	
	public static boolean isPrimitive(final Class<?> klass) {
		return primitives.contains(klass);
	}
	
	@TypeConverterMethod(byte.class)
	public static byte stringToBytePrim(final String value) {
		return Byte.parseByte(value);
	}
	
	@TypeConverterMethod(Byte.class)
	public static Byte stringToByteObj(final String value) {
		return new Byte(value);
	}
	
	@TypeConverterMethod(short.class)
	public static short stringToShortPrim(final String value) {
		return Short.parseShort(value);
	}
	
	@TypeConverterMethod(Short.class)
	public static Short stringToShortObj(final String value) {
		return new Short(value);
	}
	
	@TypeConverterMethod(int.class)
	public static int stringToIntPrim(final String value) {
		return Integer.parseInt(value);
	}
	
	@TypeConverterMethod(Integer.class)
	public static Integer stringToIntObj(final String value) {
		return new Integer(value);
	}
	
	@TypeConverterMethod(long.class)
	public static long stringToLongPrim(final String value) {
		return Long.parseLong(value);
	}
	
	@TypeConverterMethod(Long.class)
	public static Long stringToLongObj(final String value) {
		return new Long(value);
	}
	
	@TypeConverterMethod(float.class)
	public static float stringToFloatPrim(final String value) {
		return Float.parseFloat(value);
	}
	
	@TypeConverterMethod(Float.class)
	public static Float stringToFloatObj(final String value) {
		return new Float(value);
	}
	
	@TypeConverterMethod(double.class)
	public static double stringToDoublePrim(final String value) {
		return Double.parseDouble(value);
	}
	
	@TypeConverterMethod(Double.class)
	public static Double stringToDoubleObj(final String value) {
		return new Double(value);
	}
	
	@TypeConverterMethod(boolean.class)
	public static boolean stringToBooleanPrim(final String value) {
		return Boolean.parseBoolean(value);
	}
	
	@TypeConverterMethod(Boolean.class)
	public static Boolean stringToBooleanObj(final String value) {
		return new Boolean(value);
	}
	
	@TypeConverterMethod(char.class)
	public static char stringToCharPrim(final String value) {
		if ((value == null) || (value.length() == 0)) {
			return Character.MIN_VALUE;
		}
		
		return value.charAt(0);
	}
	
	@TypeConverterMethod(Character.class)
	public static Character stringToCharObj(final String value) {
		if ((value == null) || (value.length() == 0)) {
			return null;
		}
		return new Character(value.charAt(0));
	}

	private static final Collection<Class<?>> primitives = createPrimitives();

	private static Collection<Class<?>> createPrimitives() {
		final Collection<Class<?>> res = new HashSet<Class<?>>();
		
		res.add(byte.class);
		res.add(Byte.class);
		
		res.add(short.class);
		res.add(Short.class);
		
		res.add(int.class);
		res.add(Integer.class);

		res.add(long.class);
		res.add(Long.class);
		
		res.add(float.class);
		res.add(Float.class);
		
		res.add(double.class);
		res.add(Double.class);
		
		res.add(boolean.class);
		res.add(Boolean.class);
		
		res.add(char.class);
		res.add(Character.class);
		
		return res;
	}

	private PrimitiveConverter() {
	}
}
