package net.javamonkey.syringe.xml;

import net.javamonkey.syringe.models.Argument;
import net.javamonkey.syringe.models.Dependency;
import net.javamonkey.syringe.models.Factory;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author Matthew
 *
 */
final class XmlFactoryBuilder implements XmlChildBuilder {

	/* (non-Javadoc)
	 * @see net.javamonkey.syringe.xml.XmlChildBuilder#buildChild(net.javamonkey.syringe.models.Dependency, org.w3c.dom.Element)
	 */
	@Override
	public void buildChild(Dependency dependency, Element element) {
		/*
		 * <factory class="net.javamonkey.syringe.test.utils.FactoryItemFactory" method="createFactoryItem">
	 		<factory-args value="firstArg" />
	 		<factory-args value="5.678" />
	 	</factory>
		 */
		
		final Factory factory = new Factory();
		
		final String className = element.getAttribute("class");
		final String methodName = element.getAttribute("method");

		factory.setClassName(className);
		factory.setMethodName(methodName);
		
		final NodeList children = element.getChildNodes();
		final int numChildren = children.getLength();
		
		for (int i = 0; i < numChildren; ++i) {
			final Node node = children.item(i);
			if (!(node instanceof Element)) {
				continue;
			}
			
			final String nodeName = node.getNodeName();
			if (!(nodeName.equals("factory-args"))) {
				continue;
			}
			
			final Element argElem = (Element)node;
			
			final Argument arg = new Argument();
			final int argIdx;
			if (factory.hasArguments()) {
				argIdx = factory.getArguments().size();
			} else {
				argIdx = 0;
			}
			
			arg.setIndex(argIdx);
			arg.setStrValue(argElem.getAttribute("value"));
			
			factory.addArgument(arg);
		}
		
		dependency.setFactory(factory);
	}
}