/**
 * 
 */
package net.javamonkey.syringe.test;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import net.javamonkey.syringe.Syringe;
import net.javamonkey.syringe.test.utils.ConstructorItem;
import net.javamonkey.syringe.test.utils.FactoryItem;
import net.javamonkey.syringe.test.utils.FactoryItemFactory;
import net.javamonkey.syringe.test.utils.PropertyItem;
import net.javamonkey.syringe.test.utils.SingletonItem;
import net.javamonkey.syringe.test.utils.TestItem;
import net.javamonkey.syringe.xml.XmlContextLoader;

/**
 * Tests for the main Syringe class.
 * 
 * @author Matthew Cory
 *
 */
public class SyringeTests {
	
	@BeforeClass
	public static void setUpClass() {
		Syringe.registerContextLoader(new XmlContextLoader());
	}
	
	@Before
	public void setUp() {
		Syringe.loadContexts("./target/test-classes/di-sample.xml");
	}
	
	@After
	public void tearDown() {
		Syringe.clearContexts();
	}

	@Test
	public void getItemForTestItemShouldReturnCorrectlyConfiguredItem() {
		final TestItem item = Syringe.getItem("theItem", TestItem.class);
		assertNotNull(item);
		assertNull(item.getaString());
		assertEquals(TestItem.INT_DEFAULT_VALUE, item.getAnInt());
	}
	
	@Test
	public void getItemForPropItemShouldReturnItemWithProperlySetProperties() {
		final String expStringProp = "A String";
		final int expIntProp = 2;
		final double expDoubleProp = 1.234;
		final double allowed_delta = 0.0;
		final boolean expBoolProp = true;
		
		final PropertyItem propItem = Syringe.getItem("propItem", PropertyItem.class);
		
		assertNotNull(propItem);
		assertEquals(expStringProp, propItem.getStringProp());
		assertEquals(expIntProp, propItem.getIntProp());
		assertEquals(expDoubleProp, propItem.getDoubleProp(), allowed_delta);
		assertEquals(expBoolProp, propItem.getBooleanProp());
	}
	
	@Test
	public void getItemForCtorItemShouldReturnItemWithProperlySetProperties() {
		final String expStringProp = "A String";
		final int expIntProp = 5;
		
		final ConstructorItem ctorItem = Syringe.getItem("ctorItem", ConstructorItem.class);
		
		assertNotNull(ctorItem);
		assertEquals(expStringProp, ctorItem.getaString());
		assertEquals(expIntProp, ctorItem.getAnInt());
	}
	
	@Test
	public void getItemForFactoryItemShouldReturnItemWithProperlySetPropertiesAndFactoryMethodCalled() {
		final String expString = "firstArg";
		final double expDouble = 5.678;
		
		assertFalse(FactoryItemFactory.isFactoryMethodCalled());
		
		final FactoryItem factItem = Syringe.getItem("factoryItem", FactoryItem.class);
		
		assertTrue(FactoryItemFactory.isFactoryMethodCalled());
		FactoryItemFactory.resetFactoryMethodCalled();
		
		assertNotNull(factItem);
		assertEquals(expString, factItem.getaString());
		assertEquals(expDouble, factItem.getaDouble(), 0.0);
	}
	
	@Test
	public void getItemForSingletonItemShouldReturnSameInstanceEvenIfCalledMultipleTimes() {
		
		final SingletonItem firstItem = Syringe.getItem("singletonItem", SingletonItem.class);
		assertNotNull(firstItem);
		assertEquals(1, firstItem.getCount());
		
		final SingletonItem secondItem = Syringe.getItem("singletonItem", SingletonItem.class);
		assertNotNull(secondItem);
		assertEquals(1, secondItem.getCount());
		
		assertSame(firstItem, secondItem);
	}
}
