/**
 * 
 */
package net.javamonkey.syringe.test.xml;

import static org.junit.Assert.*;

import java.io.File;
import java.util.Collection;

import org.junit.Before;
import org.junit.Test;

import net.javamonkey.syringe.models.Context;
import net.javamonkey.syringe.models.Dependency;
import net.javamonkey.syringe.test.LoggedTest;
import net.javamonkey.syringe.xml.XmlContextLoader;

/**
 * @author Matthew Cory
 *
 */
public class XmlContextLoaderTests extends LoggedTest {
	
	@Before
	public void setUp() {
		ctxLdr = new XmlContextLoader();
	}
	
	@Test
	public void loadContextShouldReturnNullIfFileDoesNotExist() {
		final String missingFile = "not-a-real-file.xml";
		
		final Context ctx = ctxLdr.loadContext(missingFile);
		assertNull(ctx);
	}
	
	@Test
	public void loadContextShouldProperlyFillContextIfFileDoesExist() {
		final String contextFile = "target/test-classes/di-sample.xml";
		final File file = new File(contextFile);
		assertTrue(file.exists());
		
		final Context ctx = ctxLdr.loadContext(contextFile);
		assertNotNull(ctx);
		
		final Collection<Dependency> dependencies = ctx.getDependencies();
		
		assertNotNull(dependencies);
		
		final int expNumDependencies = 5;
		assertEquals(expNumDependencies, dependencies.size());
	}

	/**
	 * Test method for {@link net.javamonkey.syringe.xml.XmlContextLoader#canLoad(java.lang.String)}.
	 */
	@Test
	public void testCanLoad() {
		final String xmlCtxId = "valid.xml";
		final String nonXmlCtxId = "notValid.properties";
		
		assertTrue(ctxLdr.canLoad(xmlCtxId));
		assertFalse(ctxLdr.canLoad(nonXmlCtxId));
	}

	private XmlContextLoader ctxLdr;
}
